"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.roundBuffer = exports.verifyBeacon = void 0;
const bls = __importStar(require("@noble/bls12-381"));
const index_1 = require("./index");
async function verifyBeacon(chainInfo, beacon) {
    const publicKey = chainInfo.public_key;
    let message;
    if ((0, index_1.isChainedBeacon)(beacon, chainInfo)) {
        message = await chainedBeaconMessage(beacon);
    }
    else if ((0, index_1.isUnchainedBeacon)(beacon, chainInfo)) {
        message = await unchainedBeaconMessage(beacon);
    }
    else {
        console.error(`Beacon type ${chainInfo.schemeID} was not supported`);
        return false;
    }
    const signatureVerifies = await bls.verify(beacon.signature, message, publicKey);
    if (!(signatureVerifies && await randomnessIsValid(beacon))) {
        console.error('Beacon returned was invalid');
        return false;
    }
    return true;
}
exports.verifyBeacon = verifyBeacon;
async function chainedBeaconMessage(beacon) {
    const message = Buffer.concat([
        signatureBuffer(beacon.previous_signature),
        roundBuffer(beacon.round)
    ]);
    return await bls.utils.sha256(message);
}
async function unchainedBeaconMessage(beacon) {
    return await bls.utils.sha256(roundBuffer(beacon.round));
}
function signatureBuffer(sig) {
    return Buffer.from(sig, 'hex');
}
function roundBuffer(round) {
    const buffer = Buffer.alloc(8);
    buffer.writeBigUInt64BE(BigInt(round));
    return buffer;
}
exports.roundBuffer = roundBuffer;
async function randomnessIsValid(beacon) {
    const expectedRandomness = await bls.utils.sha256(Buffer.from(beacon.signature, 'hex'));
    return Buffer.from(beacon.randomness, 'hex').compare(expectedRandomness) == 0;
}
//# sourceMappingURL=beacon-verification.js.map